@@ -1,17 +1,50 @@
+# TODO: I am in the middle of a big refactor.
+# The goal is to have a Python class in the ./models
+# folder that can be used for input/output for all of
+# the functions in the connector instead of using bespoke
+# objects as we do now, which give no help to users, who
+# need to know exactly waht data to submit and who would like
+# to know what sort of data will be returned.
+
+# TODO: We should add a full testing suite for the connector.
+# See ../../tests/test_documents.py
+
+
+# region General Imports
 from io import BytesIO
-from typing import Any, Union
+from typing import Any, Union, List, Dict, Type, TypeVar
 from lxml import html
 import requests
 from requests.packages.urllib3.util.retry import Retry
 import logging
 import websocket
+import dataclasses
 import json
 
+# endregion
+
+# region LocalImports
+from .dtos import (
+    GetDocument,
+    GetDocuments,
+    PostDocument,
+    PutDocument,
+    GetAnnotationTaxonomy,
+    GetAnnotationTaxonomies,
+    PostAnnotationTaxonomy,
+    PagenatedResponse,
+    GetUser,
+)
+
+# endregion
+
+# region HTTP Adapter
 
 # Default timeouts for http requests
 # See https://findwork.dev/blog/advanced-usage-python-requests-timeouts-retries-hooks/
 from requests.adapters import HTTPAdapter
 
+
 DEFAULT_HTTP_TIMEOUT = 5  # seconds
 
 
@@ -30,18 +63,73 @@ class TimeoutHTTPAdapter(HTTPAdapter):
         return super().send(request, **kwargs)
 
 
+# endregion
+
 logger = logging.getLogger(__name__)
 
+# Typing info
+P = TypeVar("P", bound=PagenatedResponse)
+
+# JSON dataclass support (See: https://stackoverflow.com/questions/51286748/make-the-python-json-encoder-support-pythons-new-dataclasses)
+class EnhancedJSONEncoder(json.JSONEncoder):
+    def default(self, o):
+        if dataclasses.is_dataclass(o):
+            return dataclasses.asdict(o)
+        return super().default(o)
+
 
 class EscriptoriumConnector:
+    # region Init
     def __init__(
         self,
         base_url: str,
-        api_url: str,
         username: str,
         password: str,
+        api_url: str = None,
         project: str = None,
     ):
+        """Simplified access to eScriptorium
+
+        The eScriptorium connector is a class that enables convenient access to
+        an online instance of the eScriptorium platform's HTTP API. After creating an
+        EscriptoriumConnector object, the object can be used to interact with the API
+        by means of the various functions it provides.
+
+        Args:
+            base_url (str): The base url of the eScriptorium server you wish to connect to (trailing / is optional)
+            username (str): The username used to logon to the eScriptorium website
+            password (str): The password used to logon to the eScriptorium website
+            api_url (str, optional): The url path to the api (trailing / is optional). Defaults to {base_url}api/
+            project (str, optional): The name of the eScriptorium project to use by default. Defaults to None.
+
+        Raises:
+            EscriptoriumConnectorHttpError: A description of the error returned from the eScriptorium HTTP API
+
+        Examples:
+            Creating the connector and performing operations:
+
+            >>> from escriptorium_connector import EscriptoriumConnector
+            >>> from escriptorium_connector.dtos import (
+            ...     PostDocument,
+            ...     ReadDirection,
+            ...     LineOffset,
+            ... )
+
+            >>> url = "https://www.escriptorium.fr"
+            >>> username = "my_username"
+            >>> password = "my_password"
+            >>>
+            >>> connector = EscriptoriumConnector(url, username, password)
+            >>> new_project_name = "test_project"
+            >>> user_data = connector.get_user()
+            >>> user_id = user_data.count
+            >>> new_project = connector.create_project({"name": new_project_name, "slug": new_project_name, "owner": user_id})
+            >>> new_document = PostDocument(
+            ...     "test-doc", project_name, "Latin", ReadDirection.LTR, LineOffset.BASELINE, []
+            ... )
+            >>> my_doc = connector.create_document(new_document)
+        """
+
         # Setup retries and timeouts for HTTP requests
         retry_strategy = Retry(
             total=5,
@@ -58,17 +146,27 @@ class EscriptoriumConnector:
             backoff_factor=1,
         )
         adapter = TimeoutHTTPAdapter(max_retries=retry_strategy)
-        assert_status_hook = (
-            lambda response, *args, **kwargs: response.raise_for_status()
-        )
+
+        def assert_status_hook(response, *args, **kwargs):
+            try:
+                response.raise_for_status()
+            except requests.HTTPError as err:
+                EscriptoriumConnectorHttpError(err.response.text, err)
+
         self.http = requests.Session()
         self.http.hooks["response"] = [assert_status_hook]
         self.http.mount("https://", adapter)
         self.http.mount("http://", adapter)
 
         # Make sure the urls terminates with a front slash
-        self.api_url = api_url if api_url[-1] == "/" else api_url + "/"
         self.base_url = base_url if base_url[-1] == "/" else base_url + "/"
+        self.api_url = (
+            f"""{base_url}api/"""
+            if api_url is None
+            else api_url
+            if api_url[-1] == "/"
+            else f"""{api_url}/"""
+        )
 
         self.http.headers.update({"Accept": "application/json"})
 
@@ -99,7 +197,9 @@ class EscriptoriumConnector:
         self.project_name = project
         self.project = self.get_project_pk_by_name(self.project_name)
 
-    # Start websocket methods (probably not needed)
+    # endregion
+
+    # region Websockets (not used)
     def __on_message(self, ws, message):
         logging.debug(message)
 
@@ -114,35 +214,49 @@ class EscriptoriumConnector:
     def __on_open(self, ws):
         logging.debug("### websocket opened ###")
 
-    # End websocket methods (probably not needed)
+    # endregion
 
-    # Start http methods
+    # region HTTP calls
     def __get_url(self, url: str) -> requests.Response:
         return self.http.get(url)
 
     def __post_url(
-        self, url: str, payload: object, files: object = None
+        self, url: str, payload: dict, files: object = None
     ) -> requests.Response:
+        prepared_payload = json.loads(json.dumps(payload, cls=EnhancedJSONEncoder))
         return (
-            self.http.post(url, data=payload, files=files)
+            self.http.post(url, data=prepared_payload, files=files)
             if files is not None
-            else self.http.post(url, data=payload)
+            else self.http.post(url, json=prepared_payload)
         )
 
     def __put_url(
-        self, url: str, payload: object, files: object = None
+        self, url: str, payload: dict, files: object = None
     ) -> requests.Response:
+        prepared_payload = json.loads(json.dumps(payload, cls=EnhancedJSONEncoder))
         return (
-            self.http.put(url, data=payload, files=files)
+            self.http.put(url, data=prepared_payload, files=files)
             if files is not None
-            else self.http.put(url, data=payload)
+            else self.http.put(url, json=prepared_payload)
         )
 
     def __delete_url(self, url: str) -> requests.Response:
         return self.http.delete(url)
 
-    # End http methods
+    def __get_paginated_response(self, url: str, cls: Type[P]) -> P:
+        r = self.__get_url(url)
+        all_docs = cls(**json.loads(r.text))
+        info = all_docs
+        while info.next is not None:
+            r = self.__get_url(info.next)
+            info = cls(**json.loads(r.text))
+            all_docs.results = all_docs.results + info.results
 
+        return all_docs
+
+    # endregion
+
+    # region Project API
     def set_connector_project_by_name(self, project_name: str):
         self.project_name = project_name
         self.project = self.get_project_pk_by_name(self.project_name)
@@ -176,21 +290,48 @@ class EscriptoriumConnector:
         matching_projects = [x for x in all_projects if x["name"] == project_name]
         return matching_projects[0]["id"] if matching_projects else None
 
-    def get_documents(self):
-        r = self.__get_url(f"{self.api_url}documents/")
-        info = r.json()
-        documents = info["results"]
-        while info["next"] is not None:
-            r = self.__get_url(info["next"])
-            info = r.json()
-            documents = documents + info["results"]
+    def create_project(self, project_data: object):
+        return self.__post_url(f"{self.api_url}projects/", project_data)
 
-        return documents
+    def update_project(self, project_data: object):
+        return self.__put_url(f"{self.api_url}projects/", project_data)
 
-    def get_document(self, pk: int):
+    def delete_project(self, project_pk: int):
+        return self.__delete_url(f"{self.api_url}projects/{project_pk}")
+
+    def verify_project_exists(self, project_pk):
+        try:
+            self.get_project(project_pk)
+            return True
+        except:
+            return False
+
+    # endregion
+
+    # region Document API
+    def get_documents(self) -> GetDocuments:
+        return self.__get_paginated_response(f"{self.api_url}documents/", GetDocuments)
+
+    def get_document(self, pk: int) -> GetDocument:
         r = self.__get_url(f"{self.api_url}documents/{pk}/")
-        return r.json()
+        return GetDocument(**r.json())
 
+    def create_document(self, doc_data: PostDocument) -> GetDocument:
+        r = self.__post_url(f"{self.api_url}documents/", doc_data.__dict__)
+        r_json = r.json()
+        return GetDocument(**r_json)
+
+    def update_document(self, doc_data: PutDocument) -> GetDocument:
+        r = self.__put_url(f"{self.api_url}documents/", doc_data.__dict__)
+        r_json = r.json()
+        return GetDocument(**r_json)
+
+    def delete_document(self, pk: int):
+        return self.__delete_url(f"{self.api_url}documents/{pk}")
+
+    # endregion
+
+    # region Part API
     def get_document_parts(self, doc_pk: int):
         return self.get_document_images(doc_pk)
 
@@ -198,98 +339,29 @@ class EscriptoriumConnector:
         r = self.__get_url(f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/")
         return r.json()
 
-    def get_document_part_line(self, doc_pk: int, part_pk: int, line_pk: int):
-        r = self.__get_url(
-            f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/lines/{line_pk}/"
-        )
-        return r.json()
+    def delete_document_parts(self, document_pk: int, start: int, end: int):
+        parts = self.get_document_images(document_pk)
+        for part in parts[start:end]:
+            r = self.__delete_url(
+                f'{self.api_url}documents/{document_pk}/parts/{part["pk"]}/'
+            )
 
-    def get_document_part_region(self, doc_pk: int, part_pk: int, region_pk: int):
-        regions = self.get_document_part_regions(doc_pk, part_pk)
-        region = [x for x in regions if x["pk"] == region_pk]
-        return region[0] if region else None
+    # endregion
 
-    def get_document_part_line_transcription(
-        self, doc_pk: int, part_pk: int, line_pk: int, line_transcription_pk: int
-    ):
-        transcriptions = self.get_document_part_line_transcriptions(
-            doc_pk, part_pk, line_pk
-        )
-        transcription = [x for x in transcriptions if x["pk"] == line_transcription_pk]
-        return transcription[0] if transcription else None
-
-    def get_document_part_line_transcription_by_transcription(
-        self, doc_pk: int, part_pk: int, line_pk: int, transcription_pk: int
-    ):
-        transcriptions = self.get_document_part_line_transcriptions(
-            doc_pk, part_pk, line_pk
-        )
-        transcription = [
-            x for x in transcriptions if x["transcription"] == transcription_pk
-        ]
-        return transcription[0] if transcription else None
-
-    def get_document_part_line_transcriptions(
-        self, doc_pk: int, part_pk: int, line_pk: int
-    ):
-        line = self.get_document_part_line(doc_pk, part_pk, line_pk)
-        return line["transcriptions"]
-
-    def get_document_part_regions(self, doc_pk: int, part_pk: int):
-        r = self.__get_url(f"{self.api_url}documents/{doc_pk}/parts/{part_pk}")
-        part = r.json()
-        return part["regions"]
-
-    def get_document_transcription(self, doc_pk: int, transcription_pk: int):
-        r = self.__get_url(
-            f"{self.api_url}documents/{doc_pk}/transcriptions/{transcription_pk}"
-        )
-        return r.json()
-
-    def create_document_transcription(self, doc_pk: int, transcription_name: str):
-        r = self.__post_url(
-            f"{self.api_url}documents/{doc_pk}/transcriptions/",
-            {"name": transcription_name},
-        )
-        return r.json()
-
-    def get_document_transcriptions(self, doc_pk: int):
-        r = self.__get_url(f"{self.api_url}documents/{doc_pk}/transcriptions/")
-        return r.json()
-
-    def create_document_line_transcription(
-        self,
-        doc_pk: int,
-        parts_pk: int,
-        line_pk: int,
-        transcription_pk: int,
-        transcription_content: str,
-        graphs: Union[Any, None],
-    ):
-        payload = {
-            "line": line_pk,
-            "transcription": transcription_pk,
-            "content": transcription_content,
-        }
-        if graphs is not None:
-            payload["graphs"] = graphs
-        r = self.__post_url(
-            f"{self.api_url}documents/{doc_pk}/parts/{parts_pk}/transcriptions/",
-            payload,
-        )
-        return r.json()
+    # region Up/Download API
 
+    # region Text Up/Download
     def download_part_alto_transcription(
         self,
         document_pk: int,
-        part_pk: Union[list[int], int],
+        part_pk: Union[List[int], int],
         transcription_pk: int,
     ) -> Union[bytes, None]:
         """Download one or more ALTO/XML files from the document.
 
         Args:
             document_pk (int): Desired document
-            part_pk (Union[list[int], int]): Desired document part or parts
+            part_pk (Union[List[int], int]): Desired document part or parts
             transcription_pk (int): The desired transcription
 
         Returns:
@@ -305,14 +377,14 @@ class EscriptoriumConnector:
     def download_part_pagexml_transcription(
         self,
         document_pk: int,
-        part_pk: Union[list[int], int],
+        part_pk: Union[List[int], int],
         transcription_pk: int,
     ) -> Union[bytes, None]:
         """Download one or more PageXML files from the document.
 
         Args:
             document_pk (int): Desired document
-            part_pk (Union[list[int], int]): Desired document part or parts
+            part_pk (Union[List[int], int]): Desired document part or parts
             transcription_pk (int): The desired transcription
 
         Returns:
@@ -328,14 +400,14 @@ class EscriptoriumConnector:
     def download_part_text_transcription(
         self,
         document_pk: int,
-        part_pk: Union[list[int], int],
+        part_pk: Union[List[int], int],
         transcription_pk: int,
     ) -> Union[bytes, None]:
         """Download one or more TXT files from the document.
 
         Args:
             document_pk (int): Desired document
-            part_pk (Union[list[int], int]): Desired document part or parts
+            part_pk (Union[List[int], int]): Desired document part or parts
             transcription_pk (int): The desired transcription
 
         Returns:
@@ -351,7 +423,7 @@ class EscriptoriumConnector:
     def __download_part_output_transcription(
         self,
         document_pk: int,
-        part_pk: Union[list[int], int],
+        part_pk: Union[List[int], int],
         transcription_pk: int,
         output_type: str,
     ) -> Union[bytes, None]:
@@ -430,6 +502,46 @@ class EscriptoriumConnector:
             {"upload_file": (filename, file_data)},
         )
 
+    # endregion
+
+    # region Image Up/Download
+
+    def get_document_images(self, document_pk: int):
+        r = self.__get_url(f"{self.api_url}documents/{document_pk}/parts/")
+        image_info = r.json()
+        image_names = image_info["results"]
+        while image_info["next"] is not None:
+            r = self.__get_url(image_info["next"])
+            image_info = r.json()
+            image_names = image_names + image_info["results"]
+
+        return image_names
+
+    def get_image(self, img_url: str):
+        r = self.__get_url(f"{self.base_url}{img_url}")
+        return r.content
+
+    def create_image(
+        self, document_pk: int, image_data_info: object, image_data: bytes
+    ):
+        return self.__post_url(
+            f"{self.api_url}documents/{document_pk}/parts/",
+            image_data_info,
+            {"image": (image_data_info["filename"], image_data)},
+        )
+
+    # endregion
+
+    # endregion
+
+    # region Line API
+
+    def get_document_part_line(self, doc_pk: int, part_pk: int, line_pk: int):
+        r = self.__get_url(
+            f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/lines/{line_pk}/"
+        )
+        return r.json()
+
     def get_document_part_lines(self, doc_pk: int, part_pk: int):
         r = self.__get_url(f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/lines/")
         line_info = r.json()
@@ -453,6 +565,146 @@ class EscriptoriumConnector:
         )
         return r
 
+    # endregion
+
+    # region Line Type API
+
+    def get_line_types(self):
+        r = self.__get_url(f"{self.api_url}types/line/")
+        line_type_info = r.json()
+        line_types = line_type_info["results"]
+        while line_type_info["next"] is not None:
+            r = self.__get_url(line_type_info["next"])
+            line_type_info = r.json()
+            line_types = line_types + line_type_info["results"]
+
+        return line_types
+
+    def create_line_type(self, line_type: object):
+        r = self.__post_url(f"{self.api_url}types/line/", line_type)
+        return r
+
+    # endregion
+
+    # region Region API
+
+    def get_document_part_region(self, doc_pk: int, part_pk: int, region_pk: int):
+        regions = self.get_document_part_regions(doc_pk, part_pk)
+        region = [x for x in regions if x["pk"] == region_pk]
+        return region[0] if region else None
+
+    def get_document_part_regions(self, doc_pk: int, part_pk: int):
+        r = self.__get_url(f"{self.api_url}documents/{doc_pk}/parts/{part_pk}")
+        part = r.json()
+        return part["regions"]
+
+    def create_document_part_region(self, doc_pk: int, part_pk: int, region: object):
+        if not isinstance(region["box"], str):
+            region["box"] = json.dumps(region["box"])
+
+        r = self.__post_url(
+            f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/blocks/",
+            region,
+        )
+        return r
+
+    # endregion
+
+    # region Region Type API
+
+    def get_region_types(self):
+        r = self.__get_url(f"{self.api_url}types/block/")
+        block_type_info = r.json()
+        block_types = block_type_info["results"]
+        while block_type_info["next"] is not None:
+            r = self.__get_url(block_type_info["next"])
+            block_type_info = r.json()
+            block_types = block_types + block_type_info["results"]
+
+        return block_types
+
+    def get_document_region_types(self, document_pk: int) -> List[Dict[str, int]]:
+        doc_data = self.get_document(document_pk)
+        return [x for x in doc_data["valid_block_types"]]
+
+    def create_region_type(self, region_type: object):
+        r = self.__post_url(f"{self.api_url}types/block/", region_type)
+        return r
+
+    # endregion
+
+    # region Transcription API
+    def get_document_part_line_transcription(
+        self, doc_pk: int, part_pk: int, line_pk: int, line_transcription_pk: int
+    ):
+        transcriptions = self.get_document_part_line_transcriptions(
+            doc_pk, part_pk, line_pk
+        )
+        transcription = [x for x in transcriptions if x["pk"] == line_transcription_pk]
+        return transcription[0] if transcription else None
+
+    def get_document_part_line_transcription_by_transcription(
+        self, doc_pk: int, part_pk: int, line_pk: int, transcription_pk: int
+    ):
+        transcriptions = self.get_document_part_line_transcriptions(
+            doc_pk, part_pk, line_pk
+        )
+        transcription = [
+            x for x in transcriptions if x["transcription"] == transcription_pk
+        ]
+        return transcription[0] if transcription else None
+
+    def get_document_part_line_transcriptions(
+        self, doc_pk: int, part_pk: int, line_pk: int
+    ):
+        line = self.get_document_part_line(doc_pk, part_pk, line_pk)
+        return line["transcriptions"]
+
+    def get_document_transcription(self, doc_pk: int, transcription_pk: int):
+        r = self.__get_url(
+            f"{self.api_url}documents/{doc_pk}/transcriptions/{transcription_pk}/"
+        )
+        return r.json()
+
+    def create_document_transcription(self, doc_pk: int, transcription_name: str):
+        r = self.__post_url(
+            f"{self.api_url}documents/{doc_pk}/transcriptions/",
+            {"name": transcription_name},
+        )
+        return r.json()
+
+    def delete_document_transcription(self, doc_pk: int, transcription_pk: int):
+        r = self.__delete_url(
+            f"{self.api_url}documents/{doc_pk}/transcriptions/{transcription_pk}/"
+        )
+        return r.json()
+
+    def get_document_transcriptions(self, doc_pk: int):
+        r = self.__get_url(f"{self.api_url}documents/{doc_pk}/transcriptions/")
+        return r.json()
+
+    def create_document_line_transcription(
+        self,
+        doc_pk: int,
+        parts_pk: int,
+        line_pk: int,
+        transcription_pk: int,
+        transcription_content: str,
+        graphs: Union[Any, None],
+    ):
+        payload = {
+            "line": line_pk,
+            "transcription": transcription_pk,
+            "content": transcription_content,
+        }
+        if graphs is not None:
+            payload["graphs"] = graphs
+        r = self.__post_url(
+            f"{self.api_url}documents/{doc_pk}/parts/{parts_pk}/transcriptions/",
+            payload,
+        )
+        return r.json()
+
     def get_document_part_transcriptions(self, doc_pk: int, part_pk: int):
         get_url = f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/transcriptions/"
         r = self.__get_url(get_url)
@@ -473,104 +725,74 @@ class EscriptoriumConnector:
         )
         return r
 
-    def create_document_part_region(self, doc_pk: int, part_pk: int, region: object):
-        if not isinstance(region["box"], str):
-            region["box"] = json.dumps(region["box"])
+    # endregion
 
+    # region Annotation API
+    def get_document_annotations(self, doc_pk: int) -> GetAnnotationTaxonomies:
+        return self.__get_paginated_response(
+            f"""{self.api_url}documents/{doc_pk}/taxonomies/annotations/""",
+            GetAnnotationTaxonomies,
+        )
+
+    def get_document_annotation(
+        self, doc_pk: int, annotation_pk
+    ) -> GetAnnotationTaxonomy:
+        r = self.__get_url(
+            f"""{self.api_url}documents/{doc_pk}/taxonomies/annotations/{annotation_pk}""",
+        )
+        r_json = r.json()
+        return GetAnnotationTaxonomy(**r_json)
+
+    def create_document_annotation(
+        self, doc_pk: int, annotation: PostAnnotationTaxonomy
+    ) -> GetAnnotationTaxonomy:
         r = self.__post_url(
-            f"{self.api_url}documents/{doc_pk}/parts/{part_pk}/blocks/",
-            region,
+            f"{self.api_url}documents/{doc_pk}/taxonomies/annotations/",
+            annotation.__dict__,
         )
-        return r
+        r_json = r.json()
+        print(r_json)
+        return GetAnnotationTaxonomy(**r_json)
 
-    def get_line_types(self):
-        r = self.__get_url(f"{self.api_url}types/line/")
-        line_type_info = r.json()
-        line_types = line_type_info["results"]
-        while line_type_info["next"] is not None:
-            r = self.__get_url(line_type_info["next"])
-            line_type_info = r.json()
-            line_types = line_types + line_type_info["results"]
+    def update_document_annotation(
+        self, doc_pk: int, annotation_pk, annotation: PostAnnotationTaxonomy
+    ) -> GetAnnotationTaxonomy:
+        r = self.__put_url(
+            f"{self.api_url}documents/{doc_pk}/taxonomies/annotations/{annotation_pk}",
+            annotation.__dict__,
+        )
+        r_json = r.json()
+        return GetAnnotationTaxonomy(**r_json)
 
-        return line_types
-
-    def create_line_type(self, line_type: object):
-        r = self.__post_url(f"{self.api_url}types/line/", line_type)
-        return r
-
-    def get_region_types(self):
-        r = self.__get_url(f"{self.api_url}types/block/")
-        block_type_info = r.json()
-        block_types = block_type_info["results"]
-        while block_type_info["next"] is not None:
-            r = self.__get_url(block_type_info["next"])
-            block_type_info = r.json()
-            block_types = block_types + block_type_info["results"]
-
-        return block_types
-
-    def get_document_region_types(self, document_pk: int) -> list[dict[str, int]]:
-        doc_data = self.get_document(document_pk)
-        return [x for x in doc_data["valid_block_types"]]
-
-    def create_region_type(self, region_type: object):
-        r = self.__post_url(f"{self.api_url}types/block/", region_type)
-        return r
-
-    def get_document_images(self, document_pk: int):
-        r = self.__get_url(f"{self.api_url}documents/{document_pk}/parts/")
-        image_info = r.json()
-        image_names = image_info["results"]
-        while image_info["next"] is not None:
-            r = self.__get_url(image_info["next"])
-            image_info = r.json()
-            image_names = image_names + image_info["results"]
-
-        return image_names
-
-    def delete_document_parts(self, document_pk: int, start: int, end: int):
-        parts = self.get_document_images(document_pk)
-        for part in parts[start:end]:
-            r = self.__delete_url(
-                f'{self.api_url}documents/{document_pk}/parts/{part["pk"]}/'
-            )
-
-    def get_image(self, img_url: str):
-        r = self.__get_url(f"{self.base_url}{img_url}")
-        return r.content
-
-    def create_project(self, project_data: object):
-        return self.__post_url(f"{self.api_url}projects/", project_data)
-
-    def update_project(self, project_data: object):
-        return self.__put_url(f"{self.api_url}projects/", project_data)
-
-    def delete_project(self, project_pk: int):
-        return self.__delete_url(f"{self.api_url}projects/{project_pk}")
-
-    def create_document(self, doc_data: object):
-        if self.project_name and (
-            "project_name" not in doc_data or doc_data["project_name"] == ""
-        ):
-            doc_data["project"] = self.project_name
-        return self.__post_url(f"{self.api_url}documents/", doc_data)
-
-    def create_image(
-        self, document_pk: int, image_data_info: object, image_data: bytes
-    ):
-        return self.__post_url(
-            f"{self.api_url}documents/{document_pk}/parts/",
-            image_data_info,
-            {"image": (image_data_info["filename"], image_data)},
+    def delete_document_annotation(self, doc_pk: int, annotation_pk):
+        self.__delete_url(
+            f"{self.api_url}documents/{doc_pk}/taxonomies/annotations/{annotation_pk}"
         )
 
-    def verify_project_exists(self, project_pk):
-        try:
-            self.get_project(project_pk)
-            return True
-        except:
-            return False
+    # endregion
 
+    # region User API
+    def get_user(self) -> GetUser:
+        return self.__get_paginated_response(f"""{self.api_url}user""", GetUser)
+
+    # endregion
+
+
+# region Package Errors
+class EscriptoriumConnectorHttpError(BaseException):
+    def __init__(self, error_message: str, http_error: requests.HTTPError):
+        self.django_error = error_message
+        self.error = http_error
+
+    def __str__(self):
+        return self.django_error
+
+    def __repr__(self):
+        nl = "\n"
+        return f"""{self.django_error}{nl}{repr(self.error)}"""
+
+
+# endregion
 
 if __name__ == "__main__":
     import os
